@@ -1,11 +1,12 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "denote-explore" "4.0"
+(define-package "denote-explore" "20251011.232"
   "Explore and visualise Denote files."
-  '((emacs  "29.1")
-    (denote "4.0")
-    (dash   "2.19.1"))
+  '((emacs         "29.1")
+    (denote        "4.0")
+    (dash          "2.19.1")
+    (denote-regexp "20250415.2202"))
   :url "https://github.com/pprevos/denote-explore/"
-  :commit "bd5b6db435b93a2c6da694b196b4b41c5b17f68a"
-  :revdesc "bd5b6db435b9"
+  :commit "0fb33ff9e67af05aca5f8effec733713ee5f7dc4"
+  :revdesc "0fb33ff9e67a"
   :authors '(("Peter Prevos" . "peter@prevos.net"))
   :maintainers '(("Peter Prevos" . "peter@prevos.net")))
