@@ -4,9 +4,9 @@
 ;;
 ;; Author: Peter Prevos <peter@prevos.net>
 ;; URL: https://github.com/pprevos/denote-explore/
-;; Package-Version: 4.0
-;; Package-Revision: bd5b6db435b9
-;; Package-Requires: ((emacs "29.1") (denote "4.0") (dash "2.19.1"))
+;; Package-Version: 20251011.232
+;; Package-Revision: 0fb33ff9e67a
+;; Package-Requires: ((emacs "29.1") (denote "4.0") (dash "2.19.1") (denote-regexp "20250415.2202"))
 ;;
 ;; This file is NOT part of GNU Emacs.
 ;;
@@ -47,6 +47,7 @@
 (require 'cl-lib)
 (require 'json)
 (require 'browse-url)
+(require 'denote-regexp)
 
 ;;; CUSTOMISATION
 
@@ -182,6 +183,13 @@ See graphviz.org for detailed documentation."
 	  (const :tag "Portable Network Graphics (PNG)" "png")
 	  (string :tag "Other option")))
 
+(defcustom denote-explore-isolated-ignore-keywords (list)
+  "List of keywords to ignore when identifying isolated notes."
+  :group 'denote-explore
+  :type '(choice
+          (repeat :tag "List of Tags" (string :tag "Tag:"))
+          (const :tag "No Tags Ignored" nil)))
+
 ;;; INTERNAL VARIABLES
 
 (defvar denote-explore-network-graph-formats
@@ -266,9 +274,9 @@ Count only ATTACHMENTS by prefixing with universal argument."
 (defun denote-explore-count-keywords ()
   "Count distinct Denote keywords."
   (interactive)
-  (let ((all-keywords (length (mapcan #'denote-extract-keywords-from-path
-				      (denote-directory-files))))
-	(distinct-keywords (length (denote-keywords))))
+  (let* ((keywords (denote-infer-keywords-from-files))
+	 (all-keywords (length keywords))
+	 (distinct-keywords (length (delete-dups keywords))))
     (message "%s used keywords (%s distinct keywords)"
 	     all-keywords distinct-keywords)))
 
@@ -302,10 +310,12 @@ Exclude riles matching regx in `denote-explore-random-regex-ignore'.
 - `denote-explore-random-note': Jump to a random Denote file.
 - `denote-explore-random-regex': Jump to a random Denote file that matches a
   regular expression.
+- `denote-explore-random-keyword': Jump to a random Denote file with the
+  selected keyword(s).
+- `denote-explore-random-signature': Jump to a random Denote file with matching
+  signature (or sequence when using Denote-Sequence).
 - `denote-explore-random-link': Jump to a random linked note (either forward or
-  backward) or attachments (forward only).
-- `denote-explore-random-keyword': Jump to a random Denote file with the same
-  selected keyword(s)."
+  backward) or attachments (forward only)."
   (let* ((ignore (if denote-explore-random-regex-ignore
 		     (denote-directory-files denote-explore-random-regex-ignore)
 		   nil))
@@ -343,7 +353,7 @@ Uses front matter for notes and the filename for attachments."
   (let* ((file (if (eq file nil) "" file))
 	 (filetype (denote-filetype-heuristics file))
          (raw-keywords (if (denote-file-is-note-p file)
-                           (denote-retrieve-keywords-value file filetype)
+                           (denote-retrieve-front-matter-keywords-value file filetype)
                          (denote-retrieve-filename-keywords file)))
          (keywords (cond ((or (null raw-keywords) (equal raw-keywords "")) nil)
                          ((stringp raw-keywords) (split-string raw-keywords "_"))
@@ -393,13 +403,25 @@ Use Universal Argument to INCLUDE-ATTACHMENTS"
       (denote-explore--jump sample)
     (message "No matching Denote files found")))
 
+(defun denote-explore-random-signature (signature &optional include-attachments)
+  "Jump to a random note with selected SIGNATURE or sequence.
+With universal argument the sample will INCLUDE-ATTACHMENTS."
+  (interactive "sPart of signature or sequence: \nP")
+  (denote-explore-random-regex (concat "==" signature)))
+
 ;;; JANITOR
 ;; The Janitor provides various functions to maintain a Denote file collection.
 
 (defun denote-explore--table (list)
-  "Generate an ordered frequency table from a LIST."
+  "Generate an ordered frequency table from LIST.
+Sorted first by descending count, then alphabetically by item."
   (sort (-frequencies list)
-	(lambda (a b) (> (cdr a) (cdr b)))))
+        (lambda (a b)
+          (let ((ca (cdr a))
+                (cb (cdr b)))
+            (if (/= ca cb)
+                (> ca cb)                          ; sort by count, descending
+              (string-lessp (car a) (car b)))))))  ; then by string, ascending
 
 (defun denote-explore--filter-duplicates (tally)
   "Filter duplicates from an alist TALLY with frequencies.
@@ -500,8 +522,7 @@ Duplicate files are displayed `find-dired'."
   "Select a note or attachment with a keyword that is only used once."
   (interactive)
   ;; Count keywords and find singles
-  (if-let* ((keywords (mapcan #'denote-extract-keywords-from-path
-			      (denote-directory-files)))
+  (if-let* ((keywords (denote-infer-keywords-from-files))
 	    (keywords-count (denote-explore--table keywords))
 	    (single-keywords (mapcar #'car (cl-remove-if-not
 					    (lambda (note)
@@ -537,7 +558,7 @@ Duplicate files are displayed `find-dired'."
 
 Use an empty string as new keyword to remove the selection.
 
-When selecting more than one existing keyword, all selections are renamed
+When selecting more than one existing keyword, all selected terms are renamed
 to the new version or removed.
 
 The filename is taken as the source of truth for attchments and the front matter
@@ -547,30 +568,31 @@ All open Denote note buffers should be saved for this function to work reliably.
   (interactive)
   ;; Save any open Denote files
   (save-some-buffers nil #'(lambda ()
-			     (denote-filename-is-note-p buffer-file-name)))
+			                 (denote-filename-is-note-p buffer-file-name)))
   ;; Select keywords and file candidates
-  (let* ((selected-keyword (denote-keywords-prompt "Keyword(s) to rename"))
-         (keywords-regex (mapconcat
-			  (lambda (keyword) (concat "_" keyword)) selected-keyword "\\|"))
-         (files (denote-directory-files keywords-regex))
-	 (new-keyword (read-from-minibuffer "New keyword: ")))
+  (let* ((selected-keywords (denote-keywords-prompt "Keyword(s) to rename"))
+	     (keywords-regexp (denote-regexp :keywords `(:or ,@selected-keywords)))
+         (files (denote-directory-files keywords-regexp))
+	     (new-keyword (read-from-minibuffer "New keyword: ")))
     ;; Loop through candidates
     (dolist (file files)
       (let* ((denote-rename-confirmations '(rewrite-front-matter modify-file-name))
              (denote-sort-keywords t)
-	     (denote-known-keywords nil)
-	     (current-keywords (denote-explore--retrieve-keywords file))
-	     (new-keywords (if (equal new-keyword "")
-			       (cl-set-difference current-keywords selected-keyword :test 'string=)
-			     (mapcar (lambda (keyword)
-				       (if (member keyword selected-keyword) new-keyword keyword))
-				     current-keywords)))
-	     (file-type (denote-filetype-heuristics file)))
-1        (denote-rename-file file
-	 		    (denote-retrieve-title-or-filename file file-type)
-			    (if (equal new-keywords nil) "" (delete-dups new-keywords))
-			    (or (denote-retrieve-filename-signature file) "")
-			    (denote-retrieve-filename-identifier file))))))
+	         (denote-known-keywords nil)
+	         (current-keywords (denote-explore--retrieve-keywords file))
+	         (new-keywords (if (equal new-keyword "")
+                               (cl-set-difference current-keywords selected-keywords :test 'string=)
+                             (mapcar (lambda (keyword)
+                                       (if (member keyword selected-keywords) new-keyword keyword))
+                                     current-keywords)))
+	         (id (denote-retrieve-filename-identifier file))
+	         (file-type (denote-filetype-heuristics file)))
+        (denote-rename-file file
+	 		                (denote-retrieve-title-or-filename file file-type)
+			                (if (equal new-keywords nil) "" (delete-dups new-keywords))
+			                (or (denote-retrieve-filename-signature file) "")
+                            (denote-retrieve-front-matter-date-value file file-type)
+			                id)))))
 
 (define-obsolete-function-alias
   'denote-explore--retrieve-title
@@ -579,7 +601,11 @@ All open Denote note buffers should be saved for this function to work reliably.
 ;;;###autoload
 (defun denote-explore-sync-metadata ()
   "Synchronise filenames with the metadata for all Denote notes.
-The front matter is the source of truth. Keywords are saved alphabetically.
+
+The front matter is the source of truth.
+- Keywords are saved alphabetically.
+- The identifier remains unchanged
+
 All open Denote note buffers need to be saved before invoking this function."
   (interactive)
   ;; Save open Denote notes
@@ -594,7 +620,7 @@ All open Denote note buffers need to be saved before invoking this function."
 	     (file-type (denote-filetype-heuristics file))
 	     (id (denote-retrieve-filename-identifier file))
 	     (title (denote-retrieve-front-matter-title-value file file-type))
-	     (signature (denote-retrieve-filename-signature file))
+	     (signature (denote-retrieve-front-matter-signature-value file file-type))
 	     (keywords (denote-retrieve-front-matter-keywords-value file file-type))
 	     (file-keywords (denote-keywords-combine keywords))
 	     (ext (file-name-extension file t))
@@ -663,28 +689,33 @@ Follow the links in the tables to review the suspect links."
       (denote-explore--insert-missing-links "file" missing-file-links))
     (pop-to-buffer missing-links-buffer)))
 
-(defun denote-explore--insert-missing-links (type missing-links)
+(defun denote-explore--insert-missing-links-table (type missing-links)
   "Insert table of MISSING-LINKS of TYPE (Denote or file) in an Org mode buffer."
-  (let ((plural-p (if (> (length missing-links) 1) "s" "")))
-    (insert (format "\n* Missing %s link%s\n\n" type plural-p))
-    (insert (format "%s missing %s link%s\n\n"
-		    (length missing-links) type plural-p))
-    (insert "|-\n| Source | Target |\n|-\n")
-    (dolist (link missing-links)
-      (let* ((source (cdr (assoc 'source link)))
-	     (target-raw (cdr (assoc 'target link)))
-	     (target (if (eq type "Denote")
-			 target-raw
-		       (file-name-nondirectory target-raw)))
-	     (file (car (denote-directory-files source)))
-	     (file-type (denote-filetype-heuristics file))
-	     (title (denote-retrieve-front-matter-title-value file file-type)))
-	(insert "|[[elisp:(denote-explore--review-dead-link \""
-		source "\" \"" target "\")][" title "]]")
-	(insert "|" target "|\n")))
-    (insert "|-\n"))
+  (insert "\n|-\n| Source | Target |\n|-\n")
+  (dolist (link missing-links)
+    (let* ((source (cdr (assoc 'source link)))
+	   (target-raw (cdr (assoc 'target link)))
+	   (target (if (eq type "Denote")
+		       target-raw
+		     (file-name-nondirectory target-raw)))
+	   (file (car (denote-directory-files source)))
+	   (file-type (denote-filetype-heuristics file))
+	   (title (denote-retrieve-front-matter-title-value file file-type)))
+      (insert "|[[elisp:(denote-explore--review-dead-link \""
+	      source "\" \"" target "\")][" title "]]")
+      (insert "|" target "|\n")))
+  (insert "|-\n")
   (org-table-align))
 
+(defun denote-explore--insert-missing-links (type missing-links)
+  "Create Org buffer with MISSING-LINKS of TYPE (Denote or file)."
+  (let ((plural-p (if (not (= (length missing-links) 1)) "s" "")))
+    (insert (format "\n* Missing %s link%s\n" type plural-p))
+    (insert (format "%s missing %s link%s\n"
+		    (length missing-links) type plural-p))
+    (when (> (length missing-links) 0)
+      (denote-explore--insert-missing-links-table type missing-links))))
+
 (define-obsolete-function-alias
   'denote-explore-dead-links
   'denote-explore-missing-links "3.3.2")
@@ -809,7 +840,13 @@ The universal argument includes TEXT-ONLY files in the analyis."
 (defun denote-explore--idenitfy-isolated (&optional text-only)
   "Identify Denote files without (back)links.
 Using the universal argument provides TEXT-ONLY files (excludes attachments)."
-  (let* ((files (denote-directory-files nil nil text-only))
+  (let* ((all-files (denote-directory-files nil nil text-only))
+         (files (if denote-explore-isolated-ignore-keywords
+                    (-filter (lambda (filename)
+                               (not (string-match-p (rx-to-string '(and  ?_ (eval `(or ,@denote-explore-isolated-ignore-keywords))))
+                                                    filename)))
+                             all-files)
+                  all-files))
 	 (all-ids (mapcar #'denote-retrieve-filename-identifier files))
 	 (edges (denote-explore--network-extract-edges files))
 	 (linked-ids (denote-explore--network-extract-unique-nodes edges))
@@ -820,7 +857,10 @@ Using the universal argument provides TEXT-ONLY files (excludes attachments)."
 ;;;###autoload
 (defun denote-explore-isolated-files (&optional text-only)
   "Identify Denote files without (back)links.
-Using the universal argument excludes attachments (TEXT-ONLY)."
+Using the universal argument excludes attachments (TEXT-ONLY).
+
+Files which have keywords listed in
+`denote-explore-isolated-ignore-keywords' will not be included."
   (interactive "P")
   (message "Searching for isolated files ...")
   (let ((isolated (denote-explore--idenitfy-isolated text-only)))
@@ -1337,8 +1377,8 @@ Optionally analyse TEXT-ONLY files."
 			denote-explore-network-graph-formats))
 	 (convert-fn (plist-get (cdr format) :encode))
 	 (file-extension (plist-get (cdr format) :file-extension))
-	 (file-name (concat denote-explore-network-directory
-			    denote-explore-network-filename file-extension)))
+	 (file-name (file-name-concat denote-explore-network-directory
+			    (concat denote-explore-network-filename file-extension))))
     (when (not (file-exists-p denote-explore-network-directory))
       (make-directory denote-explore-network-directory))
     (with-temp-file file-name (funcall convert-fn graph))
@@ -1486,5 +1526,49 @@ Universal argument excludes attachments from the analysis (TEXT-ONLY)."
 	     (denote-explore-network-view))
     (message "No previous network defined")))
 
+;;;###autoload
+(defun denote-explore-list-keywords (&optional alphabetically)
+  "List all Denote keywords with the number of notes that use each keyword.
+
+This command:
+- Scans `denote-directory` and collects per-note keywords.
+- Builds a (KEYWORD . COUNT) table, where COUNT is the number of notes
+  that include KEYWORD (one count per note, even if repeated in metadata).
+- Sorts by COUNT descending, then KEYWORD ascending.
+- When called with universal argument, the table is sorted ALPHABETICALLY
+- When called interactively, displays a two-column table and appends
+  a footer with the total number of distinct keywords and the sum of counts.
+
+Returns a plist for programmatic use:
+  (:table ALIST :keywords N-DISTINCT :notes SUM-OF-COUNTS)."
+  (interactive "P")
+  (let* ((keywords (denote-infer-keywords-from-files))
+         ;; Use the module helper to build (KEYWORD . COUNT)
+         (table (denote-explore--table keywords))
+	 ;; If called with C-u, resort alphabetically by key
+         (table (if alphabetically
+                    (sort (copy-sequence table)
+                          (lambda (a b)
+                            (string-lessp (car a) (car b))))
+                  table))
+         (n-distinct (length table))
+         (sum-counts (apply #'+ 0 (mapcar #'cdr table))) ;; <-- FIJO AQUÍ
+         (result `(:table ,table :keywords ,n-distinct :notes ,sum-counts)))
+    (when (called-interactively-p 'interactive)
+      (with-current-buffer (get-buffer-create "*Denote Keywords*")
+        (let ((inhibit-read-only t))
+          (erase-buffer)
+          (insert (format "%-40s %s\n" "Keyword" "Notes"))
+          (insert (make-string 52 ?-) "\n")
+          (dolist (kv table)
+            (insert (format "%-40s %d\n" (car kv) (cdr kv))))
+          (insert (make-string 52 ?-) "\n")
+          (insert (format "Unique keywords: %d | Used keywords: %d\n"
+                          n-distinct sum-counts))
+          (goto-char (point-min))
+          (view-mode 1))
+        (pop-to-buffer (current-buffer))))
+    result))
+
 (provide 'denote-explore)
 ;;; denote-explore.el ends here
